import RPi.GPIO as GPIO
from server_base import GPIOControlServerBase


class GPIOControlServerRPI(GPIOControlServerBase):

    GPIO_PIN_LED = 3

    def configure_gpio(self):
        GPIO.setmode(GPIO.BOARD)
        # config LED
        GPIO.setup(self.GPIO_PIN_LED, GPIO.OUT)
        GPIO.output(self.GPIO_PIN_LED, 0)

    def write_pin(self, pin: int, value: int):
        GPIO.output(pin, value)

    def setup_pin(self, pin: int, mode):
        mode = {"input": GPIO.IN, "output": GPIO.OUT}[mode]
        GPIO.setup(pin, mode)

    def read_pin(self, pin: int) -> int:
        return GPIO.input(pin)

    def led_on(self):
        self.write_pin(self.GPIO_PIN_LED, 1)

    def led_off(self):
        self.write_pin(self.GPIO_PIN_LED, 0)



import re
import zlib
from base64 import b64encode
from io import BytesIO
from pathlib import Path
from zipfile import ZIP_DEFLATED, ZipFile

PATH_SERVER_CODE = Path(__file__).parents[1] / "server"


def _find_code_version(file_content: str) -> str:
    res = re.search(r"\s+_?VERSION = \"(.+)\"", file_content)
    if not res:
        raise ValueError("Could not find version in file")

    return res.group(1)


def _create_zip_from_files(file_paths: list[Path]) -> bytes:
    buffer = BytesIO()
    with ZipFile(buffer, "w", ZIP_DEFLATED) as zipf:
        for file_path in file_paths:
            zipf.write(file_path)

    buffer.seek(0)
    return buffer.getvalue()


def get_local_version() -> str:
    return _find_code_version((PATH_SERVER_CODE / "server_base.py").read_text())


def get_update_files(compress: bool = True) -> dict[str, str]:
    def process_file(fpath: Path) -> str:
        contents = fpath.read_text()
        if compress:
            contents = zlib.compress(contents.encode())
            contents = b64encode(contents).decode()

        return contents

    # mapping remote_location -> local_location
    files = {
        "main.py": PATH_SERVER_CODE / "main.py",
        "server_base.py": PATH_SERVER_CODE / "server_base.py",
        "server_pico.py": PATH_SERVER_CODE / "server_pico.py",
    }
    # read local files
    file_contents = {rpath: process_file(lpath) for rpath, lpath in files.items()}

    return file_contents



"""
Server-side script for automatic plant watering system.
"""

import argparse
import json
import logging
import socket
import ssl
import time
from contextlib import contextmanager
from pathlib import Path
from typing import Any, Callable

from update import get_local_version, get_update_files


class Client:

    VERSION = "1.5.0"

    CMD_TYPE = dict[str, Any]

    def __init__(self, path_config: str | Path = None, use_ssl: bool = True):
        if path_config is None:
            path_config = Path("config") / "config_connection.json"

        with open(path_config) as fh:
            config = json.load(fh)

        self.client = config["client_address"]
        self.port = config["port"]
        if use_ssl:
            self.ssl_context = ssl.create_default_context()
            self.ssl_context.check_hostname = False
            self.ssl_context.verify_mode = ssl.CERT_NONE
        else:
            self.ssl_context = None

    @contextmanager
    def _create_socket(self):
        with socket.create_connection((self.client, self.port)) as sock:
            if self.ssl_context:
                with self.ssl_context.wrap_socket(sock) as ss:
                    yield ss
            else:
                yield sock

    def _send_single_command(
        self,
        sock: socket.socket,
        command: CMD_TYPE,
        raise_exception: bool = False,
        timeout: float = 3.0,
    ) -> CMD_TYPE:
        try:
            # renew timeout for each command
            sock.settimeout(timeout)

            command["time_sent"] = time.time_ns()
            # Add linebreak to signify end of data packet
            command = json.dumps(command) + "\n"
            logging.debug(f"sending command {command}")
            sock.sendall(command.encode())
            response = self._receive_response(sock, raise_exception)
        except Exception as e:
            logging.error(f"client-side error for command {command}: {e}")
            response = {
                "error": str(e),
                "exception": e.__class__.__name__,
                "command": command,
            }

        return response

    def _receive_response(self, sock: socket.socket, raise_exception: bool) -> CMD_TYPE:
        buffer = bytearray()
        while True:
            data = sock.recv(1024)
            if not data:
                # means server closed the connection
                break

            buffer.extend(data)

        response = json.loads(buffer.decode())
        response["time_responded"] = time.time_ns()
        if "error" in response:
            logfun = logging.critical if raise_exception else logging.error
            logfun(f"server-side error: {response}")
            if raise_exception:
                # TODO
                raise NotImplementedError("refactor this try/except block")
                # raise __builtins__[response['exception']](response['error'])

        return response

    def send_commands(
        self, cmds: list[CMD_TYPE], raise_exception: bool = False, timeout: float = 3.0
    ) -> list[CMD_TYPE]:
        """Send several commands through a single connection."""
        responses = []
        with self._create_socket() as sock:
            # logging.debug(f'connected to {self.client} on port {self.port}')
            for command in cmds:
                response = self._send_single_command(
                    sock, command, raise_exception, timeout
                )
                responses.append(response)

        return responses

    def send_command(self, raise_exception: bool = False, **kwargs) -> CMD_TYPE:
        return self.send_commands([kwargs], raise_exception=raise_exception)[0]

    def send_ping(self, n: int = 5):
        def avg(lst: list[float]) -> float:
            return sum(lst) / len(lst)

        responses = self.send_commands([dict(action="ping") for _ in range(n)])
        error = any("error" in r for r in responses)
        # compute stats
        rtt, tsend = [], []
        for r in responses:
            rtt.append(r["time_responded"] - r["time_sent"])
            tsend.append(r["time_received"] - r["time_sent"])

        return {"rtt": avg(rtt) / 1e9, "tsend": avg(tsend) / 1e9, "error": error}

    def poll_command_response(
        self,
        cmd: CMD_TYPE,
        fun_validate: Callable[[CMD_TYPE], bool],
        dt: float,
        timeout: float,
        allow_connection_refused: bool = False,
    ) -> bool:
        # TODO: refactor this with send_commands to avoid code redundancy
        assert dt > 0 and timeout > 0
        try:
            with self._create_socket() as sock:
                start = time.time()
                while time.time() - start < timeout:
                    response = self._send_single_command(sock, cmd)

                    if fun_validate(response):
                        return True

                    time.sleep(dt)
        except ConnectionRefusedError:
            if allow_connection_refused:
                time.sleep(dt)
                return self.poll_command_response(
                    cmd, fun_validate, dt, timeout, allow_connection_refused
                )

            raise

        return False

    def update_server(self, compress_transfer: bool = True):
        info = self.send_command(action="get_info")["info"]
        print("Server info:")
        print(json.dumps(info, indent=2))
        if "pico" not in info["board"].lower():
            raise NotImplementedError("Server is not running on a Pico board")

        server_version = self.send_command(action="get_version")["version"]
        print("Server version:", server_version)

        local_version = get_local_version()
        if local_version == server_version:
            print("Server is up to date")
            return
        print("Local version:", local_version)

        files: dict = get_update_files(compress=compress_transfer)
        print("Files to update:", list(files.keys()))
        sumchar = sum(map(len, files.values()))
        print("Payload size:", sumchar, "bytes")

        self.send_command(action="update", files=files, compress=compress_transfer)
        print("Waiting for server to restart")
        time.sleep(3)
        print("Attempting to reconnect")
        connected = self.poll_command_response(
            dict(action="ping"),
            lambda x: x.get("action") == "ping",
            1,
            10,
            allow_connection_refused=True,
        )
        if not connected:
            print("Could not connect to server after update")
            return

        new_version = self.send_command(action="get_version")["version"]
        if new_version != local_version:
            print("Update failed, current server version:", new_version)
            return

        print("Server updated successfully")


def main(args):
    use_ssl = not args.no_ssl
    client = Client(use_ssl=use_ssl)

    assert not (args.command and args.file), "Cannot send both a command and a file"

    if args.update:
        client.update_server()
        return

    if args.command:
        res = client.send_commands(args.command, timeout=args.timeout)
    elif args.file:
        with args.file as f:
            commands = json.load(f)
        res = client.send_commands(commands, timeout=args.timeout)
    else:
        res = client.send_ping()

    print("Response:")
    print(json.dumps(res, indent=2))


class KwargsAppendAction(argparse.Action):
    """
    Argparse action to split an argument into KEY=VALUE form
    on append to a list of dictionaries.
    """

    def __call__(self, parser, args, values, option_string=None):
        try:
            d = dict(map(lambda x: x.split("="), values))
        except ValueError:
            raise argparse.ArgumentError(
                self, f'Could not parse argument "{values}" as k1=v1 k2=v2 ... format'
            )

        if getattr(args, self.dest) is None:
            setattr(args, self.dest, [])

        getattr(args, self.dest).append(d)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--no-ssl",
        action="store_true",
    )
    parser.add_argument(
        "-c",
        "--command",
        nargs="*",
        required=False,
        action=KwargsAppendAction,
        metavar="KEY=VALUE",
        help="Command set to server. If not sent, sends a ping",
    )
    parser.add_argument(
        "-f",
        "--file",
        type=argparse.FileType("r"),
        help="File containing commands to send to the server",
        required=False,
    )
    parser.add_argument(
        "--timeout",
        type=float,
        default=3.0,
        help="TCP socket timeout in seconds (for each command)",
    )
    parser.add_argument(
        "--update",
        action="store_true",
        help="Update server and ignore other arguments",
    )
    args = parser.parse_args()
    print(args)
    main(args)



"""Top-level package for piconetcontrol."""

__author__ = """Matthias Gilles Zeller"""
__author_email__ = "matthias.gilles.zeller@gmail.com"
__version__ = "0.2.1"
__description__ = "Client-server package to remotely control a Raspberrypi Pi Pico W."
__url__ = "https://github.com/matthiaszeller/picoserver"



import gc
import json
import os
from io import BytesIO
from time import sleep

import machine
import network
from deflate import DeflateIO
from server_base import GPIOControlServerBase, GPIOPinNotSetupError
from ubinascii import a2b_base64


class GPIOControlServerPicoW(GPIOControlServerBase):

    def __init__(
        self,
        path_wifi_credentials: str,
        port: int,
        path_ssl_cert: str = None,
        path_ssl_key: str = None,
    ):
        self.led = machine.Pin("LED", machine.Pin.OUT)
        super().__init__(port, path_ssl_cert, path_ssl_key)
        self.pins = dict()

        with open(path_wifi_credentials, "r") as fh:
            credentials = json.load(fh)

        self.wlan_ssid = credentials["ssid"]
        self.wlan_pwd = credentials["pwd"]

    def configure_network(self):
        wlan = network.WLAN(network.STA_IF)
        if not wlan.isconnected():
            wlan.active(True)
            wlan.connect(self.wlan_ssid, self.wlan_pwd)

            while not wlan.isconnected():
                print("waiting for connection...")
                self.blink_led(100, 3, 200)
                sleep(1)

    def led_on(self):
        self.led.on()

    def led_off(self):
        self.led.off()

    def __process_pin(self, pin: int, check_setup=True):
        # attempt casting to int
        try:
            pin = int(pin)
        except ValueError:
            pass

        if check_setup and pin not in self.pins:
            raise GPIOPinNotSetupError(f"pin {pin} not setup")

        return pin

    def _decompress(self, file_contents: str) -> str:
        with DeflateIO(BytesIO(a2b_base64(file_contents))) as f:
            return f.read().decode()

    def setup_pin(self, pin: int, mode):
        mode = {"input": machine.Pin.IN, "output": machine.Pin.OUT}[mode]
        pin = self.__process_pin(pin, check_setup=False)
        self.pins[pin] = machine.Pin(pin, mode)

    def write_pin(self, pin: int, value: int):
        pin = self.__process_pin(pin)
        self.pins[pin].value(value)

    def read_pin(self, pin: int) -> int:
        pin = self.__process_pin(pin)
        return self.pins[pin].value()

    def get_info(self) -> dict:
        uname = os.uname()
        return {
            "mem_free": gc.mem_free(),
            "mem_alloc": gc.mem_alloc(),
            "micropython_version": uname.release,
            "micropython_version_info": uname.version,
            "board": uname.machine,
        }

    def sleep(self, time_ms: int, deep: bool):
        # make sure board stops blinking
        self._event_continuous_blink.clear()
        self.led_off()

        if deep:
            print(f"deepsleeping for {time_ms} ms...")
            machine.deepsleep(time_ms)
        else:
            print(f"lightsleeping for {time_ms} ms...")
            machine.lightsleep(time_ms)

        # to see the print appear, wait just a bit with time.sleep (software-based),
        # or it might not show up (probably depends on how microcontroller handles prints
        # when interrupts occur)
        sleep(0.1)
        print("woke up from sleep")
        self._event_continuous_blink.set()

    def reset_after_timeout(self, soft: bool, timeout_ms: int = 1000):
        if soft:
            print("soft resetting...")
            fun = machine.soft_reset
        else:
            print("resetting...")
            fun = machine.reset

        machine.Timer().init(
            period=timeout_ms, mode=machine.Timer.ONE_SHOT, callback=lambda t: fun()
        )



from pathlib import Path

from setuptools import find_packages, setup


def read_package_metadata(
    package_path: str | Path = None,
) -> dict[str, str]:
    def parse_line(line: str) -> tuple[str, str] | None:
        assignment = line.split("=", 1)
        if len(assignment) == 2:
            var, value = assignment
            var = var.strip().strip("_")
            value = value.strip().strip("'\"")
            return var, value

    if package_path is None:
        package_path = Path(__file__).parent
        package_path = package_path / package_path.name

    with open(package_path / "__init__.py") as f:
        content = f.read()

    fields = dict()
    for line in content.splitlines():
        res = parse_line(line)
        if res:
            fields[res[0]] = res[1]

    return fields


package_metadata = read_package_metadata()
print(package_metadata)

setup(
    name="piconetcontrol",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Hardware",
        "Topic :: System :: Networking",
    ],
    # python_requires=">=3.11",
    long_description=Path("README.rst").read_text(),
    long_description_content_type="text/x-rst",
    **package_metadata,
)



"""
Client-side script for operating GPIO pins on the Rasperry Pi Pico W.
Communication adheres to a defined communication protocol, see `README.md`.
"""

import asyncio
import json
import os
import ssl

# time.time() in micropython has no sub-second precision
from time import sleep
from time import time_ns as time

import machine

# micropython has no sleep_ms
try:
    from time import sleep_ms
except ImportError:

    def sleep_ms(time_ms: int):
        sleep(time_ms / 1000)


def is_raspberrypi_pico() -> bool:
    try:
        import machine  # noqa F401

        return True
    except ImportError:
        return False


def json_decorator(fun):
    async def wrapper(*args, **kwargs) -> str:
        try:
            dic = json.loads(args[1])  # args[0] is self
            args = list(args)
            args[1] = dic
            try:
                res = await fun(*args, **kwargs)
            except Exception as e:
                res = dic.copy()
                print("error in function", fun.__name__, e)
                res["error"] = e

        # MicroPython throws ValueError, Python a JSONDecodeError
        # but JSONDecodeError is a subclass of ValueError
        except ValueError as e:
            print("error decoding JSON", e)
            res = {"error": e}

        if "error" in res:
            res["exception"] = res["error"].__class__.__name__
            res["error"] = str(res["error"])

        return json.dumps(res)

    return wrapper


def update_file(file_path: str, content: str):
    # if file already exists, rename it for backup
    try:
        os.rename(file_path, "_" + file_path)
    except OSError:
        # nothing to do if file doesn't exist
        pass

    with open(file_path, "w") as f:
        f.write(content)


class GPIOControlServerBase:

    _VERSION = "1.13.0"

    _IDLING_BLINK_DURATION = 1.5
    _IDLING_BLINK_DT = 1.5

    def __init__(
        self,
        port: int,
        path_ssl_cert: str = None,
        path_ssl_key: str = None,
    ):
        self.connection_port = port

        assert (path_ssl_cert is None) == (
            path_ssl_key is None
        ), "both or none of ssl_cert and ssl_key must be provided"
        if path_ssl_cert:
            self.ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
            self.ssl_context.load_cert_chain(path_ssl_cert, path_ssl_key)
        else:
            self.ssl_context = None

        self._event_continuous_blink = asyncio.Event()
        self._event_continuous_blink.set()
        self.__continuous_blink_duration = None
        self.__continuous_blink_dt = None

    async def run(self):
        self.configure_gpio()
        self.configure_network()
        task_blink = asyncio.create_task(self.__blink_led_infinite())
        try:
            await self.server_listen()
        finally:
            task_blink.cancel()

    def cleanup(self):
        # critical to correctly shut down LED if code has terminated (e.g. error)
        self.led_off()

    def configure_network(self):
        pass

    def configure_gpio(self):
        pass

    def blink_led(self, duration_ms: int = 50, n: int = 1, dt: int = 50):
        for _ in range(n):
            self.led_on()
            sleep_ms(duration_ms)
            self.led_off()
            sleep_ms(dt)

    async def blink_led_async(
        self, duration_s: float = 0.05, n: int = 3, dt_s: float = 0.1
    ):
        # TODO: could use machine.Timer instead, but be careful with infinite blink
        #       could probably play with Timer.init and Timer.deinit
        try:
            self._event_continuous_blink.clear()  # pause infinite blinking
            for _ in range(n):
                self.led_on()
                await asyncio.sleep(duration_s)
                self.led_off()
                await asyncio.sleep(dt_s)
        finally:
            self._event_continuous_blink.set()  # resume infinite blinking

    def update_continuous_blink(self, duration_s: float = None, dt_s: float = None):
        self.__continuous_blink_duration = duration_s or self._IDLING_BLINK_DURATION
        self.__continuous_blink_dt = dt_s or self._IDLING_BLINK_DT

    async def __blink_led_infinite(self, duration_s: float = None, dt_s: float = None):
        self.update_continuous_blink(duration_s, dt_s)

        while True:
            await self._event_continuous_blink.wait()
            self.led_on()
            await asyncio.sleep(self.__continuous_blink_duration)
            self.led_off()
            await asyncio.sleep(self.__continuous_blink_dt)

    def led_on(self):
        pass

    def led_off(self):
        pass

    async def handle_client(
        self, reader: "asyncio.StreamReader", writer: "asyncio.StreamWriter"
    ):
        print("agent connected", reader.get_extra_info("peername"))
        task_blink = asyncio.create_task(self.blink_led_async(0.020, 100000, 0.1))
        try:
            buffer = bytearray()
            while True:
                data = await reader.read(1024)
                if not data:
                    # No data means the client has closed the connection
                    break

                buffer.extend(data)

                # check if we have received a full command
                if data.endswith(b"\n"):
                    print("received data", buffer)
                    response = await self.handle_command(buffer.decode())
                    writer.write(response.encode())
                    await writer.drain()
                    # exit loop after sending response
                    break

        finally:
            task_blink.cancel()
            writer.close()
            await writer.wait_closed()

    async def server_listen(self):
        addr = "0.0.0.0"
        server = await asyncio.start_server(
            self.handle_client, addr, self.connection_port, ssl=self.ssl_context
        )
        print("listening on", addr, "port", self.connection_port)
        async with server:
            await server.wait_closed()

    def setup_pin(self, pin: int, mode):
        pass

    def write_pin(self, pin: int, value: int):
        pass

    def read_pin(self, pin: int) -> int:
        pass

    def get_info(self) -> dict:
        pass

    def sleep(self, time_ms: int, deep: bool):
        pass

    def reset_after_timeout(self, soft: bool, timeout_ms: int = 1000):
        pass

    def reset(self, soft: bool):
        pass

    @staticmethod
    def _validate_command(command: dict, *fields: tuple[str, type]):
        missing_fields = set(varname for varname, _ in fields).difference(
            command.keys()
        )
        if len(missing_fields) > 0:
            raise ValueError(
                f'incomplete command, missing fields: {", ".join(missing_fields)}'
            )

        res = (
            [fieldtype(command[field]) for field, fieldtype in fields]
            if len(fields) > 1
            else fields[0][1](command[fields[0][0]])
        )
        return res

    def _action_setup_pin(self, command: dict):
        pin, mode = self._validate_command(command, ("pin", int), ("mode", str))
        self.setup_pin(pin, mode)
        # optionally set pin value
        if "value" in command:
            value = self._validate_command(command, ("value", int))
            self.write_pin(pin, value)

    def _action_write_pin(self, command: dict):
        pin, value, timeout = self._validate_command(
            command, ("pin", int), ("value", int), ("timeout", float)
        )
        # check if pin already has desired value
        if self.read_pin(pin) == value:
            return

        self.write_pin(pin, value)
        # reset pin after timeout
        reset_value = 1 - value
        asyncio.create_task(self.write_pin_after_timeout(pin, reset_value, timeout))

    def _action_read_pin(self, command: dict):
        pin = self._validate_command(command, ("pin", int))
        command["value"] = self.read_pin(pin)

    def _action_ping(self, command: dict):
        pass

    def _action_reset(self, command: dict):
        self.reset_after_timeout(soft=False)

    def _action_sleep(self, command: dict):
        time_ms, deep = self._validate_command(command, ("time_ms", int), ("deep", int))
        # sleep after some time to allow response to be sent
        machine.Timer().init(
            mode=machine.Timer.ONE_SHOT,
            period=1000,
            callback=lambda t: self.sleep(time_ms, deep),
        )

    # Disable soft reset: main.py doesn't seem to run after soft reset
    # def _action_soft_reset(self, command: dict):
    #     asyncio.create_task(self.reset_after_timeout(soft=True))

    def _action_get_version(self, command: dict):
        command["version"] = self._VERSION

    def _action_get_info(self, command: dict):
        command["info"] = self.get_info()

    def _action_list_actions(self, command: dict):
        command["actions"] = list(self._ACTIONS.keys())

    def _decompress(self, file_contents: str) -> str:
        pass

    def _action_update(self, command: dict):
        files, compress = self._validate_command(
            command, ("files", dict), ("compress", bool)
        )
        # decompress
        if compress:
            files = {k: self._decompress(v) for k, v in files.items()}

        # signal update is in progress
        with open("update.txt", "w") as f:
            f.write(",".join(files.keys()))

        for k, v in files.items():
            print("writing file", k, len(v))
            update_file(k, v)

        self.reset_after_timeout(soft=False)

    @json_decorator
    async def handle_command(self, command: dict[str]) -> dict[str]:
        command = command.copy()
        command["time_received"] = time()

        fun = self._ACTIONS.get(command["action"])
        if fun is None:
            raise ValueError(
                f'unknown action "{command["action"]}", use "list_actions" to list available actions'
            )

        fun(self, command)

        return command

    async def write_pin_after_timeout(self, pin: int, value: int, timeout: float):
        # TODO replace async callback with machine.Timer
        await asyncio.sleep(timeout)
        print("resetting pin", pin, "to value", value)
        self.write_pin(pin, value)

    _ACTIONS = {
        "setup_pin": _action_setup_pin,
        "write_pin": _action_write_pin,
        "read_pin": _action_read_pin,
        "ping": _action_ping,
        # "soft_reset": _action_soft_reset,
        "reset": _action_reset,
        "get_version": _action_get_version,
        "get_info": _action_get_info,
        "sleep": _action_sleep,
        "list_actions": _action_list_actions,
        "update": _action_update,
    }


class GPIOPinNotSetupError(RuntimeError):
    pass



import asyncio
import os

import machine


def restore_file(file_path: str):
    print("restoring file", file_path)
    os.rename("_" + file_path, file_path)


try:
    from server_base import GPIOControlServerBase, is_raspberrypi_pico

    if is_raspberrypi_pico():
        from server_pico import GPIOControlServerPicoW

        Server = GPIOControlServerPicoW
        args = {
            "port": 12345,
            "path_wifi_credentials": "config/config_wlan.json",
            "path_ssl_cert": "config/ec_cert.der",
            "path_ssl_key": "config/ec_key.der",
        }

    else:
        from server_rpi import GPIOControlServerRPI

        Server = GPIOControlServerRPI

    async def main(app: GPIOControlServerBase):
        await app.run()

    if __name__ == "__main__":
        app = Server(**args)

        try:
            asyncio.run(main(app))
        finally:
            app.cleanup()

except Exception:
    # check if update has just been made
    try:
        with open("update.txt", "r") as fh:
            updated = fh.read().split(",")
    except Exception:
        # no update just made
        updated = None

    if updated:
        with open("logs.txt", "w") as fh:
            fh.write("update failed, attempting rollback...")

        for file in updated:
            restore_file(file)

        os.remove("update.txt")
        with open("logs.txt", "a") as fh:
            fh.write("rollback successful... resetting")

        machine.reset()
